package kr.co.vividnext.sodalive.admin.calculate

import com.querydsl.core.annotations.QueryProjection
import java.math.BigDecimal
import java.math.RoundingMode

data class GetCalculateCommunityPostQueryData @QueryProjection constructor(
    val nickname: String,
    val title: String,
    val date: String,
    val can: Int,
    val numberOfPurchase: Long,
    val totalCan: Int,
    val settlementRatio: Int?
) {
    fun toGetCalculateCommunityPostResponse(): GetCalculateCommunityPostResponse {
        // 원화 = totalCoin * 100 ( 캔 1개 = 100원 )
        val totalKrw = BigDecimal(totalCan).multiply(BigDecimal(100))

        // 결제수수료 : 6.6%
        val paymentFee = totalKrw.multiply(BigDecimal(0.066))

        // 정산금액 = (원화 - 결제수수료) 의 70%
        val settlementAmount = if (settlementRatio != null) {
            totalKrw.subtract(paymentFee).multiply(BigDecimal(settlementRatio).divide(BigDecimal(100.0)))
        } else {
            totalKrw.subtract(paymentFee).multiply(BigDecimal(0.7))
        }

        // 원천세 = 정산금액의 3.3%
        val tax = settlementAmount.multiply(BigDecimal(0.033))

        // 입금액
        val depositAmount = settlementAmount.subtract(tax)

        return GetCalculateCommunityPostResponse(
            nickname = nickname,
            title = title,
            date = date,
            can = can,
            numberOfPurchase = numberOfPurchase.toInt(),
            totalCan = totalCan,
            totalKrw = totalKrw.toInt(),
            paymentFee = paymentFee.setScale(0, RoundingMode.HALF_UP).toInt(),
            settlementAmount = settlementAmount.setScale(0, RoundingMode.HALF_UP).toInt(),
            tax = tax.setScale(0, RoundingMode.HALF_UP).toInt(),
            depositAmount = depositAmount.setScale(0, RoundingMode.HALF_UP).toInt()
        )
    }
}
