package kr.co.vividnext.sodalive.admin.calculate

import com.querydsl.core.annotations.QueryProjection
import kr.co.vividnext.sodalive.content.order.OrderType
import java.math.BigDecimal
import java.math.RoundingMode

data class GetCalculateContentQueryData @QueryProjection constructor(
    // 등록 크리에이터 닉네임
    val nickname: String,
    // 콘텐츠 제목
    val title: String,
    // 콘텐츠 등록 날짜
    val registrationDate: String,
    // 콘텐츠 판매 날짜
    val saleDate: String,
    // 대여/소장 구분
    val orderType: OrderType,
    // 판매 금액(캔)
    val orderPrice: Int,
    // 인원
    val numberOfPeople: Long,
    // 합계
    val totalCan: Int,
    // 포인트
    val totalPoint: Int,
    // 정산비율
    val settlementRatio: Int?
) {
    fun toGetCalculateContentResponse(): GetCalculateContentResponse {
        val orderTypeStr = if (totalPoint > 0) {
            "포인트"
        } else if (orderType == OrderType.RENTAL) {
            "대여"
        } else {
            "소장"
        }

        // 원화 = totalCoin * 100 ( 캔 1개 = 100원 )
        val totalKrw = BigDecimal(totalCan).multiply(BigDecimal(100))

        // 결제수수료 : 6.6%
        val paymentFee = totalKrw.multiply(BigDecimal(0.066))

        // 정산금액 = (원화 - 결제수수료) 의 70%
        val settlementAmount = if (settlementRatio != null) {
            totalKrw.subtract(paymentFee).multiply(BigDecimal(settlementRatio).divide(BigDecimal(100.0)))
        } else {
            totalKrw.subtract(paymentFee).multiply(BigDecimal(0.7))
        }

        // 원천세 = 정산금액의 3.3%
        val tax = settlementAmount.multiply(BigDecimal(0.033))

        val depositAmount = settlementAmount.subtract(tax)

        return GetCalculateContentResponse(
            nickname = nickname,
            title = title,
            registrationDate = registrationDate,
            saleDate = saleDate,
            orderType = orderTypeStr,
            orderPrice = orderPrice,
            numberOfPeople = numberOfPeople.toInt(),
            totalCan = totalCan,
            totalKrw = totalKrw.toInt(),
            paymentFee = paymentFee.setScale(0, RoundingMode.HALF_UP).toInt(),
            settlementAmount = settlementAmount.setScale(0, RoundingMode.HALF_UP).toInt(),
            tax = tax.setScale(0, RoundingMode.HALF_UP).toInt(),
            depositAmount = depositAmount.setScale(0, RoundingMode.HALF_UP).toInt()
        )
    }
}
