package kr.co.vividnext.sodalive.admin.can

import kr.co.vividnext.sodalive.admin.member.AdminMemberRepository
import kr.co.vividnext.sodalive.can.CanStatus
import kr.co.vividnext.sodalive.can.charge.Charge
import kr.co.vividnext.sodalive.can.charge.ChargeRepository
import kr.co.vividnext.sodalive.can.charge.ChargeStatus
import kr.co.vividnext.sodalive.can.payment.Payment
import kr.co.vividnext.sodalive.can.payment.PaymentGateway
import kr.co.vividnext.sodalive.can.payment.PaymentStatus
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.extensions.moneyFormat
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class AdminCanService(
    private val repository: AdminCanRepository,
    private val chargeRepository: ChargeRepository,
    private val memberRepository: AdminMemberRepository
) {
    @Transactional
    fun saveCan(request: AdminCanRequest) {
        repository.save(request.toEntity())
    }

    @Transactional
    fun deleteCan(id: Long) {
        val can = repository.findByIdOrNull(id)
            ?: throw SodaException("잘못된 요청입니다.")

        can.status = CanStatus.END_OF_SALE
    }

    @Transactional
    fun charge(request: AdminCanChargeRequest) {
        val member = memberRepository.findByIdOrNull(request.memberId)
            ?: throw SodaException("잘못된 회원번호 입니다.")

        if (request.can <= 0) throw SodaException("1 캔 이상 입력하세요.")
        if (request.method.isBlank()) throw SodaException("기록내용을 입력하세요.")

        val charge = Charge(0, request.can, status = ChargeStatus.ADMIN)
        charge.title = "${request.can.moneyFormat()} 캔"
        charge.member = member

        val payment = Payment(status = PaymentStatus.COMPLETE, paymentGateway = PaymentGateway.PG)
        payment.method = request.method
        charge.payment = payment

        chargeRepository.save(charge)

        member.pgRewardCan += charge.rewardCan
    }
}
