package kr.co.vividnext.sodalive.admin.event.charge

import com.querydsl.core.types.dsl.DateTimePath
import com.querydsl.core.types.dsl.Expressions
import com.querydsl.core.types.dsl.StringTemplate
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.admin.event.charge.QChargeEvent.chargeEvent
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository
import java.time.LocalDateTime

@Repository
interface AdminChargeEventRepository : JpaRepository<ChargeEvent, Long>, AdminChargeEventQueryRepository

interface AdminChargeEventQueryRepository {
    fun getChargeEventList(): List<GetChargeEventListResponse>
}

class AdminChargeEventQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : AdminChargeEventQueryRepository {
    override fun getChargeEventList(): List<GetChargeEventListResponse> {
        return queryFactory
            .select(
                QGetChargeEventListResponse(
                    chargeEvent.id,
                    chargeEvent.title,
                    getFormattedDate(chargeEvent.startDate),
                    getFormattedDate(chargeEvent.endDate),
                    chargeEvent.availableCount,
                    chargeEvent.addPercent.multiply(100).castToNum(Int::class.java),
                    chargeEvent.isActive
                )
            )
            .from(chargeEvent)
            .orderBy(chargeEvent.createdAt.desc())
            .fetch()
    }

    private fun getFormattedDate(dateTimePath: DateTimePath<LocalDateTime>): StringTemplate {
        return Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                dateTimePath,
                "UTC",
                "Asia/Seoul"
            ),
            "%Y-%m-%d %H:%i"
        )
    }
}
