package kr.co.vividnext.sodalive.can.charge.event

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.admin.event.charge.ChargeEvent
import kr.co.vividnext.sodalive.admin.event.charge.QChargeEvent.chargeEvent
import kr.co.vividnext.sodalive.can.charge.QCharge.charge
import kr.co.vividnext.sodalive.can.payment.PaymentStatus
import kr.co.vividnext.sodalive.can.payment.QPayment.payment
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.jpa.repository.JpaRepository
import java.time.LocalDateTime

interface ChargeEventRepository : JpaRepository<ChargeEvent, Long>, ChargeEventQueryRepository

interface ChargeEventQueryRepository {
    fun getChargeEvent(): ChargeEvent?
    fun getPaymentCount(member: Member, method: String, startDate: LocalDateTime, endDate: LocalDateTime): Int
}

class ChargeEventQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : ChargeEventQueryRepository {
    override fun getChargeEvent(): ChargeEvent? {
        val now = LocalDateTime.now()
        return queryFactory
            .selectFrom(chargeEvent)
            .where(
                chargeEvent.isActive.isTrue
                    .and(chargeEvent.startDate.loe(now))
                    .and(chargeEvent.endDate.goe(now))
            )
            .orderBy(chargeEvent.id.asc())
            .fetchFirst()
    }

    override fun getPaymentCount(
        member: Member,
        method: String,
        startDate: LocalDateTime,
        endDate: LocalDateTime
    ): Int {
        val where = charge.member.eq(member)
            .and(charge.payment.method.eq(method))
            .and(
                charge.payment.status.eq(PaymentStatus.COMPLETE)
                    .or(charge.payment.status.eq(PaymentStatus.RETURN))
            )

        return queryFactory
            .selectFrom(charge)
            .innerJoin(charge.payment, payment)
            .where(where)
            .fetch()
            .count()
    }
}
