package kr.co.vividnext.sodalive.can.coupon

import com.querydsl.core.types.dsl.DateTimePath
import com.querydsl.core.types.dsl.Expressions
import com.querydsl.core.types.dsl.StringTemplate
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.can.coupon.QCanCoupon.canCoupon
import org.springframework.data.jpa.repository.JpaRepository
import java.time.LocalDateTime

interface CanCouponRepository : JpaRepository<CanCoupon, Long>, CanCouponQueryRepository

interface CanCouponQueryRepository {
    fun getCouponTotalCount(): Int
    fun getCouponList(offset: Long, limit: Long): List<GetCouponListItemResponse>
}

class CanCouponQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : CanCouponQueryRepository {
    override fun getCouponTotalCount(): Int {
        return queryFactory
            .select(canCoupon.id)
            .from(canCoupon)
            .fetch()
            .size
    }

    override fun getCouponList(offset: Long, limit: Long): List<GetCouponListItemResponse> {
        return queryFactory
            .select(
                QGetCouponListItemResponse(
                    canCoupon.id,
                    canCoupon.couponName,
                    canCoupon.can,
                    canCoupon.couponCount,
                    Expressions.ZERO,
                    getFormattedDate(canCoupon.validity),
                    canCoupon.isMultipleUse,
                    canCoupon.isActive
                )
            )
            .from(canCoupon)
            .orderBy(canCoupon.id.desc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    private fun getFormattedDate(dateTimePath: DateTimePath<LocalDateTime>): StringTemplate {
        return Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                dateTimePath,
                "UTC",
                "Asia/Seoul"
            ),
            "%Y-%m-%d"
        )
    }
}
