package kr.co.vividnext.sodalive.content.main.tab.alarm

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.domain.Pageable
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/v2/audio-content/main/alarm")
class AudioContentMainTabAlarmController(private val service: AudioContentMainTabAlarmService) {
    @GetMapping
    fun fetchContentMainTabAlarm(
        @RequestParam("isAdultContentVisible", required = false) isAdultContentVisible: Boolean? = null,
        @RequestParam("contentType", required = false) contentType: ContentType? = null,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.fetchData(
                isAdultContentVisible = isAdultContentVisible ?: true,
                contentType = contentType ?: ContentType.ALL,
                member
            )
        )
    }

    @GetMapping("/all")
    fun fetchAlarmContentByTheme(
        @RequestParam("theme") theme: String,
        @RequestParam("isAdultContentVisible", required = false) isAdultContentVisible: Boolean? = null,
        @RequestParam("contentType", required = false) contentType: ContentType? = null,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.fetchAlarmContentByTheme(
                theme,
                member,
                isAdultContentVisible = isAdultContentVisible ?: true,
                contentType = contentType ?: ContentType.ALL,
                offset = pageable.offset,
                limit = pageable.pageSize.toLong()
            )
        )
    }
}
