package kr.co.vividnext.sodalive.content.main.tab.content

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.main.GetAudioContentMainItem
import kr.co.vividnext.sodalive.content.main.QGetAudioContentMainItem
import kr.co.vividnext.sodalive.content.main.curation.tag.QContentHashTagCuration.contentHashTagCuration
import kr.co.vividnext.sodalive.content.main.curation.tag.QContentHashTagCurationItem.contentHashTagCurationItem
import kr.co.vividnext.sodalive.member.QMember.member
import kr.co.vividnext.sodalive.member.block.QBlockMember.blockMember
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Repository

@Repository
class ContentMainTabTagCurationRepository(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) {
    fun getTagList(isAdult: Boolean, contentType: ContentType): List<String> {
        var where = contentHashTagCuration.isActive.isTrue
            .and(audioContent.isActive.isTrue)
            .and(audioContent.duration.isNotNull)
            .and(audioContent.limited.isNull)
            .and(contentHashTagCurationItem.isActive.isTrue)

        if (!isAdult) {
            where = where.and(contentHashTagCuration.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    audioContent.member.isNull.or(
                        audioContent.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        return queryFactory
            .select(contentHashTagCuration.tag)
            .from(contentHashTagCuration)
            .innerJoin(contentHashTagCurationItem)
            .on(contentHashTagCurationItem.curation.id.eq(contentHashTagCuration.id))
            .innerJoin(contentHashTagCurationItem.content, audioContent)
            .where(where)
            .groupBy(contentHashTagCuration.id)
            .having(contentHashTagCurationItem.id.countDistinct().gt(0))
            .orderBy(contentHashTagCuration.orders.asc())
            .fetch()
    }

    fun getTagCurationContentList(
        memberId: Long,
        tag: String,
        contentType: ContentType
    ): List<GetAudioContentMainItem> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        var where = audioContent.isActive.isTrue
            .and(audioContent.duration.isNotNull)
            .and(audioContent.limited.isNull)
            .and(blockMember.id.isNull)
            .and(contentHashTagCurationItem.isActive.isTrue)
            .and(contentHashTagCuration.tag.eq(tag))

        if (contentType != ContentType.ALL) {
            where = where.and(
                audioContent.member.isNull.or(
                    audioContent.member.auth.gender.eq(
                        if (contentType == ContentType.MALE) {
                            0
                        } else {
                            1
                        }
                    )
                )
            )
        }

        return queryFactory
            .select(
                QGetAudioContentMainItem(
                    audioContent.id,
                    audioContent.coverImage.prepend("/").prepend(imageHost),
                    audioContent.title,
                    member.id,
                    member.profileImage.prepend("/").prepend(imageHost),
                    member.nickname,
                    audioContent.price,
                    audioContent.duration
                )
            )
            .from(contentHashTagCurationItem)
            .innerJoin(contentHashTagCurationItem.curation, contentHashTagCuration)
            .innerJoin(contentHashTagCurationItem.content, audioContent)
            .innerJoin(audioContent.member, member)
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .orderBy(contentHashTagCurationItem.orders.asc())
            .fetch()
    }
}
