package kr.co.vividnext.sodalive.explorer.profile.creatorCommunity

import kr.co.vividnext.sodalive.common.BaseEntity
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.comment.GetCommunityPostCommentListItem
import kr.co.vividnext.sodalive.member.Member
import javax.persistence.Column
import javax.persistence.Entity
import javax.persistence.FetchType
import javax.persistence.JoinColumn
import javax.persistence.ManyToOne

@Entity
data class CreatorCommunity(
    @Column(columnDefinition = "TEXT", nullable = false)
    var content: String,
    var price: Int,
    var isCommentAvailable: Boolean,
    var isAdult: Boolean,
    @Column(nullable = true)
    var audioPath: String? = null,
    @Column(nullable = true)
    var imagePath: String? = null,
    var isActive: Boolean = true
) : BaseEntity() {
    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "member_id", nullable = false)
    var member: Member? = null

    fun toCommunityPostListResponse(
        imageHost: String,
        audioUrl: String?,
        content: String,
        date: String,
        isLike: Boolean,
        existOrdered: Boolean,
        likeCount: Int,
        commentCount: Int,
        firstComment: GetCommunityPostCommentListItem?
    ): GetCommunityPostListResponse {
        return GetCommunityPostListResponse(
            postId = id!!,
            creatorId = member!!.id!!,
            creatorNickname = member!!.nickname,
            creatorProfileUrl = "$imageHost/${member?.profileImage ?: "profile/default-profile.png"}",
            imageUrl = if (imagePath != null) {
                "$imageHost/$imagePath"
            } else {
                null
            },
            audioUrl = audioUrl,
            content = content,
            price = price,
            date = date,
            isCommentAvailable = isCommentAvailable,
            isAdult = false,
            isLike = isLike,
            existOrdered = existOrdered,
            likeCount = likeCount,
            commentCount = commentCount,
            firstComment = firstComment
        )
    }
}
