package kr.co.vividnext.sodalive.marketing

import com.querydsl.core.types.dsl.DateTimePath
import com.querydsl.core.types.dsl.Expressions
import com.querydsl.core.types.dsl.StringTemplate
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.admin.marketing.GetAdminAdMediaPartnerResponseItem
import kr.co.vividnext.sodalive.admin.marketing.QGetAdminAdMediaPartnerResponseItem
import kr.co.vividnext.sodalive.marketing.QAdMediaPartner.adMediaPartner
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository
import java.time.LocalDateTime

interface AdMediaPartnerRepository : JpaRepository<AdMediaPartner, Long>, AdMediaPartnerQueryRepository

interface AdMediaPartnerQueryRepository {
    fun findByPid(pid: String): AdMediaPartner?
    fun getMediaPartnerList(offset: Long, limit: Long): List<GetAdminAdMediaPartnerResponseItem>
    fun getMediaPartnerListTotalCount(): Int
}

@Repository
class AdMediaPartnerQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : AdMediaPartnerQueryRepository {
    override fun findByPid(pid: String): AdMediaPartner? {
        return queryFactory
            .selectFrom(adMediaPartner)
            .where(adMediaPartner.pid.eq(pid), adMediaPartner.isActive.isTrue)
            .fetchFirst()
    }

    override fun getMediaPartnerList(offset: Long, limit: Long): List<GetAdminAdMediaPartnerResponseItem> {
        return queryFactory
            .select(
                QGetAdminAdMediaPartnerResponseItem(
                    adMediaPartner.id,
                    adMediaPartner.mediaGroup,
                    adMediaPartner.pid,
                    adMediaPartner.pidName,
                    adMediaPartner.type,
                    adMediaPartner.utmSource,
                    adMediaPartner.utmMedium,
                    adMediaPartner.isActive,
                    getFormattedDate(adMediaPartner.createdAt),
                    Expressions.constant("")
                )
            )
            .from(adMediaPartner)
            .orderBy(adMediaPartner.isActive.desc(), adMediaPartner.id.desc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    override fun getMediaPartnerListTotalCount(): Int {
        return queryFactory
            .select(adMediaPartner.id)
            .from(adMediaPartner)
            .fetch()
            .size
    }

    private fun getFormattedDate(dateTimePath: DateTimePath<LocalDateTime>): StringTemplate {
        return Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                dateTimePath,
                "UTC",
                "Asia/Seoul"
            ),
            "%Y-%m-%d %H:%i:%s"
        )
    }
}
