package kr.co.vividnext.sodalive.point

import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.order.QOrder.order
import kr.co.vividnext.sodalive.point.QUsePoint.usePoint
import org.springframework.data.jpa.repository.JpaRepository
import java.time.LocalDateTime

interface UsePointRepository : JpaRepository<UsePoint, Long>, UsePointQueryRepository

interface UsePointQueryRepository {
    fun getPointUseStatusByMemberId(memberId: Long, timezone: String): List<GetPointUseStatusResponse>
}

class UsePointQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : UsePointQueryRepository {
    override fun getPointUseStatusByMemberId(memberId: Long, timezone: String): List<GetPointUseStatusResponse> {
        val formattedDate = Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                usePoint.createdAt,
                "UTC",
                timezone
            ),
            "%Y.%m.%d | %H:%i:%s"
        )

        return queryFactory
            .select(
                QGetPointUseStatusResponse(
                    audioContent.title.prepend("[콘텐츠 대여] "),
                    formattedDate,
                    usePoint.amount
                )
            )
            .from(usePoint)
            .innerJoin(order).on(usePoint.orderId.eq(order.id))
            .innerJoin(audioContent).on(order.audioContent.id.eq(audioContent.id))
            .where(usePoint.memberId.eq(memberId))
            .orderBy(usePoint.id.desc())
            .fetch()
    }
}
